from django.contrib import admin
from django import forms
from .models import ZohoToken
from .zoho import ZohoClient

from django.shortcuts import render, redirect
from django.urls import reverse


class FetchTokensForm(forms.Form):
    authorization_code = forms.CharField()


class ZohoTokenAdmin(admin.ModelAdmin):
    list_display = ("access_token", "refresh_token", "timestamp")
    change_list_template = "admin/zoho_client/zohotoken/zoho_token_change_list.html"

    def get_urls(self):
        from django.urls import path

        urls = super().get_urls()
        custom_urls = [
            path(
                "fetch-tokens/",
                self.admin_site.admin_view(self.fetch_tokens),
                name="zoho_client_zohotoken_fetch_tokens",
            ),
        ]
        return custom_urls + urls

    def fetch_tokens(self, request):
        if request.method == "POST":
            form = FetchTokensForm(request.POST)
            if form.is_valid():
                code = form.cleaned_data["authorization_code"]
                ZohoClient().fetch_tokens(code)
                # redirect to the ZohoToken list page
                return redirect(reverse("admin:zoho_client_zohotoken_changelist"))
        else:
            form = FetchTokensForm()

        return render(
            request, "admin/zoho_client/zohotoken/fetch_tokens.html", {"form": form}
        )


admin.site.register(ZohoToken, ZohoTokenAdmin)



# Generated by Django 4.2.3 on 2023-07-14 12:22

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ZohoToken",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("access_token", models.TextField()),
                ("refresh_token", models.TextField()),
                ("timestamp", models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



import requests
from .models import ZohoToken
from django.conf import settings


class ZohoClient:
    def __init__(self):
        self.client_id = settings.ZOHO_CLIENT_ID
        self.client_secret = settings.ZOHO_CLIENT_SECRET
        self.token_url = "https://accounts.zoho.com/oauth/v2/token"
        self.api_version = getattr(settings, "ZOHO_API_VERSION", "v2.1")
        self.base_url = getattr(settings, "ZOHO_BASE_URL", "https://zohoapis.com")
        self.headers = {}

        try:
            token = ZohoToken.objects.latest("timestamp")
            self.refresh_token = token.refresh_token
            self.access_token = token.access_token
        except ZohoToken.DoesNotExist:
            self.refresh_token = None
            self.access_token = None

    def make_request(self, method="GET", api_endpoint=None, data=None):
        if not self.access_token:
            self.refresh_access_token()
        url = f"{self.base_url}/crm/{self.api_version}/{api_endpoint}"
        print(url)
        response = requests.request(method, url, headers=self.headers, json=data)

        # If the access token has expired, refresh it and try the request again
        if response.status_code == 401:
            self.refresh_access_token()
            return self.make_request(method, api_endpoint, data)

        return response.json()

    def refresh_access_token(self):
        data = {
            "refresh_token": self.refresh_token,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "grant_type": "refresh_token",
        }

        response = requests.post(self.token_url, data=data)
        self.access_token = response.json().get("access_token")

        # Save the new tokens to the database
        ZohoToken.objects.create(
            access_token=self.access_token, refresh_token=self.refresh_token
        )

        self.headers = {"Authorization": f"Zoho-oauthtoken {self.access_token}"}

    def fetch_tokens(self, authorization_code):
        data = {
            "code": authorization_code,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": "http://localhost",  # Or your actual redirect URI
            "grant_type": "authorization_code",
        }

        response = requests.post(self.token_url, data=data)
        response_data = response.json()

        if response.status_code == 200 and "refresh_token" in response_data:
            self.refresh_token = response_data["refresh_token"]
            self.access_token = response_data["access_token"]

            # Save the new tokens to the database
            ZohoToken.objects.create(
                access_token=self.access_token, refresh_token=self.refresh_token
            )

            return self.refresh_token

        else:
            # Handle error
            error = response_data.get("error", "Unknown error")
            raise Exception(f"Failed to fetch tokens: {error}")



from setuptools import setup, find_packages
from os import path

# Read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="zoho-client-django",
    version="1.1.0",  # Update this for new versions
    description="Django app which is a client for the Zoho CRM API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Guy Moller",
    author_email="guy.moller@gmail.com",
    url="https://github.com/gosourcellc/zoho-client-django",
    packages=find_packages(),
    install_requires=[
        "Django>=4.2.3,<5.0",
        "requests>=2.31.0,<3.0",
    ],
)
